@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -36,9 +42,11 @@
 
 #include <QtQml/qtqmlglobal.h>
 #include <QtCore/qstring.h>
+#include <QtCore/qvariant.h>
 
 QT_BEGIN_NAMESPACE
 
+#ifndef QT_NO_QML_DEBUGGER
 
 struct Q_QML_EXPORT QQmlDebuggingEnabler
 {
@@ -48,10 +56,19 @@ struct Q_QML_EXPORT QQmlDebuggingEnabler
     };
 
     QQmlDebuggingEnabler(bool printWarning = true);
+
+    static QStringList debuggerServices();
+    static QStringList inspectorServices();
+    static QStringList profilerServices();
+
+    static void setServices(const QStringList &services);
+
     static bool startTcpDebugServer(int port, StartMode mode = DoNotWaitForClient,
                                     const QString &hostName = QString());
     static bool connectToLocalDebugger(const QString &socketFileName,
                                        StartMode mode = DoNotWaitForClient);
+    static bool startDebugConnector(const QString &pluginName,
+                                    const QVariantHash &configuration = QVariantHash());
 };
 
 // Execute code in constructor before first QQmlEngine is instantiated
@@ -61,6 +78,8 @@ static QQmlDebuggingEnabler qQmlEnableDebuggingHelper(false);
 static QQmlDebuggingEnabler qQmlEnableDebuggingHelper(true);
 #endif
 
+#endif
+
 QT_END_NAMESPACE
 
 #endif // QQMLDEBUG_H
